Index: drivers/dahdi/dahdi_dynamic.c
===================================================================
--- drivers/dahdi/dahdi_dynamic.c	(revision 7668)
+++ drivers/dahdi/dahdi_dynamic.c	(working copy)
@@ -128,13 +128,14 @@
 static rwlock_t drvlock = RW_LOCK_UNLOCKED;
 #endif
 
-static void checkmaster(void)
+static void checkmaster(int lock)
 {
 	unsigned long flags;
 	int newhasmaster=0;
 	int best = 9999999;
 	struct dahdi_dynamic *z, *master=NULL;
-	spin_lock_irqsave(&dlock, flags);
+	if (lock)
+		spin_lock_irqsave(&dlock, flags);
 	z = dspans;
 	while(z) {
 		if (z->timing) {
@@ -154,7 +155,8 @@
 	/* Mark the new master if there is one */
 	if (master)
 		master->master = 1;
-	spin_unlock_irqrestore(&dlock, flags);
+	if (lock)
+		spin_unlock_irqrestore(&dlock, flags);
 	if (master)
 		printk(KERN_INFO "TDMoX: New master: %s\n", master->span.name);
 	else
@@ -395,7 +397,7 @@
 	if (newalarm != span->alarms) {
 		span->alarms = newalarm;
 		dahdi_alarm_notify(span);
-		checkmaster();
+		checkmaster(1);
 	}
 	
 	/* Keep track of last received packet */
@@ -411,7 +413,7 @@
 	
 }
 
-static void dynamic_destroy(struct dahdi_dynamic *z)
+static void dynamic_destroy(struct dahdi_dynamic *z, int lock)
 {
 	unsigned int x;
 
@@ -435,7 +437,7 @@
 	/* Free z */
 	kfree(z);
 
-	checkmaster();
+	checkmaster(lock);
 }
 
 static struct dahdi_dynamic *find_dynamic(struct dahdi_dynamic_span *zds)
@@ -493,11 +495,11 @@
 		prev = cur;
 		cur = cur->next;
 	}
+
+	/* Destroy it */
+	dynamic_destroy(z, 0);
 	spin_unlock_irqrestore(&dlock, flags);
 
-	/* Destroy it */
-	dynamic_destroy(z);
-	
 	return 0;
 }
 
@@ -531,7 +533,7 @@
 	if (z) 
 		z->usecount--;
 	if (z->dead && !z->usecount)
-		dynamic_destroy(z);
+		dynamic_destroy(z, 1);
 	return 0;
 }
 
@@ -575,7 +577,7 @@
 
 	for (x = 0; x < zds->numchans; x++) {
 		if (!(z->chans[x] = kmalloc(sizeof(*z->chans[x]), GFP_KERNEL))) {
-			dynamic_destroy(z);
+			dynamic_destroy(z, 1);
 			return -ENOMEM;
 		}
 
@@ -588,7 +590,7 @@
 	z->msgbuf = kmalloc(bufsize, GFP_KERNEL);
 
 	if (!z->msgbuf) {
-		dynamic_destroy(z);
+		dynamic_destroy(z, 1);
 		return -ENOMEM;
 	}
 	
@@ -645,7 +647,7 @@
 	   have it here */
 	if (!ztd) {
 		printk(KERN_NOTICE "No such driver '%s' for dynamic span\n", zds->driver);
-		dynamic_destroy(z);
+		dynamic_destroy(z, 1);
 		return -EINVAL;
 	}
 
@@ -663,7 +665,7 @@
 	/* Whee!  We're created.  Now register the span */
 	if (dahdi_register(&z->span, 0)) {
 		printk(KERN_NOTICE "Unable to register span '%s'\n", z->span.name);
-		dynamic_destroy(z);
+		dynamic_destroy(z, 1);
 		return -EINVAL;
 	}
 
@@ -673,7 +675,7 @@
 	dspans = z;
 	spin_unlock_irqrestore(&dlock, flags);
 
-	checkmaster();
+	checkmaster(1);
 
 	/* All done */
 	return z->span.spanno;
@@ -774,7 +776,7 @@
 			else
 				dspans = z->next;
 			if (!z->usecount)
-				dynamic_destroy(z);
+				dynamic_destroy(z, 0);
 			else
 				z->dead = 1;
 		} else {
@@ -810,7 +812,7 @@
 	}
 	spin_unlock_irqrestore(&dlock, flags);
 	if (alarmchanged)
-		checkmaster();
+		checkmaster(1);
 
 	/* Do the next one */
 	mod_timer(&alarmcheck, jiffies + 1 * HZ);
